
*==>obstacle.gms

$TITLE An Obstacle Problem
$offsymxref offsymlist offuellist offuelxref
$inlinecom /* */

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   program to calculate the position of a membrane
   pushed up through a (rectangular) hole in a rigid plate;
   in addition, there are rigid obstacle(s) inside the hole
   (perhaps not at the same level as the plate).
   These obstacles can constrain the membrane from above
   or from below.
   The correct position of the membrane (the position
   of minimum energy) is determined by the minimization
   of a quadratic function of the membrane position,
   subject to the constraints imposed by the hole
   and the obstacle.  The MCP below arises from the
   optimality conditions for this QP.  Reference:

	Ciarlet, Philippe G.  The finite element method for 
	elliptic problems.  North-Holland, 1978.

   this gams file should write a data file (memb.dat)
   which can be used to view the computed membrane.
   If you have matlab and the file viewmemb.m,
   you can see the membrane quite easily.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */

sets
I	/ 0 * 51 /,
J	/ 0 * 51 /;

scalars
xlo	/ 0 /,
xhi	/ 1 /,
ylo	/ 0 /,
yhi	/ 1 /,
dx,
dy,
c	/ 1 /;		/* force constant */

dx = (xhi - xlo) / (card(I)-1);
dy = (yhi - ylo) / (card(J)-1);

parameters
lb(I,J),
ub(I,J);

variables
v(I,J);		/* height of membrane */

equations
dv(I,J);

dv(I,J) ..
  (dy/dx) * (2*v(I,J) - v(I+1,J) - v(I-1,J))
+ (dx/dy) * (2*v(I,J) - v(I,J+1) - v(I,J-1))
=g=
c * dx * dy;

model obst	/ dv.v /;

option solprint = off;
option limrow = 0;
option limcol = 0;
option iterlim = 100000;
option reslim = 36000;
* obst.workspace = 30;

/* begin problems from Dembo & Tulowitzki */
lb(I,J) = sin(3.2*(xlo + dx*(ord(I)-1)))
	* sin(3.3*(ylo + dy*(ord(J)-1)));
ub(I,J) = 2000;
/* the position v is fixed at zero on the boundary */
ub(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
ub(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
lb(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
lb(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
v.lo(I,J) = lb(I,J);
v.up(I,J) = ub(I,J);

parameter sol1(I,J), sol2(I,J), sol3(I,J), sol4(I,J);

v.l(I,J) = 1;
solve obst using mcp;
sol1(I,J) = v.l(I,J);

lb(I,J) = power(sin(9.2*(xlo + dx*(ord(I)-1)))
	      * sin(9.3*(ylo + dy*(ord(J)-1))), 3);
ub(I,J) = power(sin(9.2*(xlo + dx*(ord(I)-1)))
	      * sin(9.3*(ylo + dy*(ord(J)-1))), 2)
	+ 0.02;
/* the position v is fixed at zero on the boundary */
ub(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
ub(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
lb(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
lb(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
v.lo(I,J) = lb(I,J);
v.up(I,J) = ub(I,J);

v.l(I,J) = (ub(I,J) + lb(I,J)) / 2;
solve obst using mcp;
sol2(I,J) = v.l(I,J);


lb(I,J) = power(16 * (xlo + dx*(ord(I)-1)) * (1 - (xlo + dx*(ord(I)-1)))
	      * (ylo + dy*(ord(J)-1)) * (1 - (ylo + dy*(ord(J)-1))),
	 3);
ub(I,J) = power(16 * (xlo + dx*(ord(I)-1)) * (1 - (xlo + dx*(ord(I)-1)))
	      * (ylo + dy*(ord(J)-1)) * (1 - (ylo + dy*(ord(J)-1))),
	 2) + 0.01;
/* the position v is fixed at zero on the boundary */
ub(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
ub(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
lb(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
lb(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
v.lo(I,J) = lb(I,J);
v.up(I,J) = ub(I,J);

v.l(I,J) = (ub(I,J) + lb(I,J)) / 2;
solve obst using mcp;
sol3(I,J) = v.l(I,J);

/* end problems from Dembo & Tulowitzki */

/* problem with a ball constraining from above */
scalar
midI, midJ,
cheight		/ .55 /,
ball_rad	/ 0.5 /;


midI = (card(I) + 1) / 2;
midJ = (card(J) + 1) / 2;
* use lb as temp 
lb(I,J) = ( power(ord(I)-midI,2) + power(ord(J)-midJ,2) ) /
		(midI*midI + midJ*midJ) ;
ub(I,J) = min (2000, cheight - ball_rad*sqrt(1-lb(I,J)) );
lb(I,J) = 0;


v.lo(I,J) = lb(I,J);
v.up(I,J) = ub(I,J);
v.l(I,J) = (ub(I,J) + lb(I,J)) / 2;
* v.l(I,J) = lb(I,J);
* v.l(I,J) = ub(I,J);

solve obst using mcp;
sol4(I,J) = v.l(I,J);

put '::sol1'/;
put '2 ' card(i):0:0 ' ' card(j):0:0/;

loop (I,
    loop (J$(abs(sol1(I,J)) gt 1e-3),
	put ord(i):0:0'.'ord(j):0:0 sol1(I,J):12:6 /;
	);
);

put '::sol2'/;
put '2 ' card(i):0:0 ' ' card(j):0:0/;

loop (I,
    loop (J$(abs(sol2(I,J)) gt 1e-3),
	put ord(i):0:0'.'ord(j):0:0 sol2(I,J):12:6 /;
	);
);

put '::sol3'/;
put '2 ' card(i):0:0 ' ' card(j):0:0/;

loop (I,
    loop (J$(abs(sol3(I,J)) gt 1e-3),
	put ord(i):0:0'.'ord(j):0:0 sol3(I,J):12:6 /;
	);
);

put '::sol4'/;
put '2 ' card(i):0:0 ' ' card(j):0:0/;

loop (I,
    loop (J$(abs(sol4(I,J)) gt 1e-3),
	put ord(i):0:0'.'ord(j):0:0 sol4(I,J):12:6 /;
	);
);
